"""Test that the ai4gh package imports correctly."""

from __future__ import annotations


def test_import() -> None:
    """Test that ai4gh can be imported."""
    import ai4gh

    assert ai4gh.__version__ == "1.0.0"


def test_core_import() -> None:
    """Test that ai4gh.core can be imported."""
    import ai4gh.core  # noqa: F401
