"""Tests for MCP server module."""

from __future__ import annotations

from ai4gh.mcp.server import server


class TestServer:
    def test_server_exists(self) -> None:
        assert server is not None

    def test_server_name(self) -> None:
        assert server.name == "ai4gh"
