"""Compliance test suites for GA4GH specifications.

Import all suites to trigger registration.
"""

from __future__ import annotations

from ai4gh.compliance.suites import beacon, drs, htsget, refget, tes, trs, wes

__all__ = ["beacon", "drs", "htsget", "refget", "tes", "trs", "wes"]
