"""GA4GH TES (Task Execution Service) v1.1 plugin."""

from __future__ import annotations

from ai4gh.specs.tes.client import TESClient
from ai4gh.specs.tes.models import (
    TESCreateTaskResponse,
    TESExecutor,
    TESInput,
    TESListTasksResponse,
    TESOutput,
    TESResources,
    TESState,
    TESTask,
    TESTaskLog,
    TESView,
)

__all__ = [
    "TESClient",
    "TESCreateTaskResponse",
    "TESExecutor",
    "TESInput",
    "TESListTasksResponse",
    "TESOutput",
    "TESResources",
    "TESState",
    "TESTask",
    "TESTaskLog",
    "TESView",
]
