"""GA4GH refget v2 plugin."""

from __future__ import annotations

from ai4gh.specs.refget.client import RefgetClient
from ai4gh.specs.refget.models import RefgetAlias, RefgetMetadata

__all__ = [
    "RefgetAlias",
    "RefgetClient",
    "RefgetMetadata",
]
